@@ -1,6 +1,8 @@
+{-# LANGUAGE DeriveFunctor #-}
+
 -- | This module provides a type and functions for handling focus rings
 -- of widgets. Note that this interface is merely provided for managing
--- the focus state for a sequence of widget names; it does not do
+-- the focus state for a sequence of resource names; it does not do
 -- anything beyond keep track of that.
 --
 -- This interface is experimental.
@@ -21,12 +23,13 @@ import Data.Maybe (listToMaybe)
 import Brick.Types
 import Brick.Widgets.Core (Named(..))
 
--- | A focus ring containing a sequence of widget names to focus and a
--- currently-focused widget name.
+-- | A focus ring containing a sequence of resource names to focus and a
+-- currently-focused name.
 data FocusRing n = FocusRingEmpty
                  | FocusRingNonempty ![n] !Int
+                 deriving Functor
 
--- | Construct a focus ring from the list of names.
+-- | Construct a focus ring from the list of resource names.
 focusRing :: [n] -> FocusRing n
 focusRing [] = FocusRingEmpty
 focusRing names = FocusRingNonempty names 0
@@ -48,11 +51,11 @@ focusPrev (FocusRingNonempty ns i) = FocusRingNonempty ns i'
         i' = (i + (length ns) - 1) `mod` (length ns)
 
 -- | This function is a convenience function to look up a widget state
--- value's name in a focus ring and set its focus setting according to
--- the focus ring's state. This function determines whether a given
--- widget state value is the focus of the ring and passes the resulting
--- boolean to a rendering function, along with the state value (a), to
--- produce whatever comes next (b).
+-- value's resource name in a focus ring and set its focus setting
+-- according to the focus ring's state. This function determines whether
+-- a given widget state value is the focus of the ring and passes the
+-- resulting boolean to a rendering function, along with the state value
+-- (a), to produce whatever comes next (b).
 --
 -- Focus-aware widgets have rendering functions that should be
 -- usable with this combinator; see 'Brick.Widgets.List.List' and
@@ -68,8 +71,8 @@ withFocusRing :: (Eq n, Named a n)
               -- ^ The rest of the computation.
 withFocusRing ring f a = f (focusGetCurrent ring == Just (getName a)) a
 
--- | Get the currently-focused widget name from the ring. If the ring is
--- emtpy, return 'Nothing'.
+-- | Get the currently-focused resource name from the ring. If the ring
+-- is emtpy, return 'Nothing'.
 focusGetCurrent :: FocusRing n -> Maybe n
 focusGetCurrent FocusRingEmpty = Nothing
 focusGetCurrent (FocusRingNonempty ns i) = Just $ ns !! i
@@ -85,8 +88,8 @@ focusRingCursor :: (Eq n)
                 -> [CursorLocation n]
                 -- ^ The list of available cursor positions.
                 -> Maybe (CursorLocation n)
-                -- ^ The cursor position, if any, that matches the name
-                -- currently focused by the 'FocusRing'.
+                -- ^ The cursor position, if any, that matches the
+                -- resource name currently focused by the 'FocusRing'.
 focusRingCursor getRing st ls =
     listToMaybe $ filter isCurrent ls
     where
